@@ -277,6 +277,7 @@ static int m920x_i2c_xfer(struct i2c_adapter *adap, struct i2c_msg msg[], int nu
 			char *read = kmalloc(1, GFP_KERNEL);
 			if (!read) {
 				ret = -ENOMEM;
+				kfree(read);
 				goto unlock;
 			}
 
@@ -287,10 +288,8 @@ static int m920x_i2c_xfer(struct i2c_adapter *adap, struct i2c_msg msg[], int nu
 
 				if ((ret = m920x_read(d->udev, M9206_I2C, 0x0,
 						      0x20 | stop,
-						      read, 1)) != 0) {
-					kfree(read);
+						      read, 1)) != 0)
 					goto unlock;
-				}
 				msg[i].buf[j] = read[0];
 			}
 
